(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    206986,       3589]*)
(*NotebookOutlinePosition[    207762,       3616]*)
(*  CellTagsIndexPosition[    207718,       3612]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[
    \(\( (*\ NGaussElimination\ *) \n (*\ 
      Formula\ for\ 3\ by\ 3\ block\ matrix\ *) \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Clear[NGaussElimination]; \)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(NGaussElimination\)\" \
is similar to existing symbol \"\!\(GaussElimination\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\( (*\ Should\ use\ or\ not\ ?? \ *) \n
    \(<< LinearAlgebra`MatrixManipulation`; \)\)\)], "Input"],

Cell[BoxData[
    \(\(NGaussElimination::usage = \n\t\t"\<put some comments\>"; \)\)], 
  "Input"],

Cell[BoxData[
    \( (*\ Without\ NCSimplifyRational\ *) \)], "Input"],

Cell[BoxData[
    \(\(NGaussElimination[mat_] := 
      Module[{output, n, \ submata, submatb, submatc, submatd, subgauss}, \ \n
        \ \ \ \ \ n = \(Dimensions[mat]\)[\([1]\)]; \n\t\t
        If[n < 3, \ \n\t\t\tReturn[GaussElimination[mat]], \ \n
          \t (*\ Else\ *) \n\t\t\t (*\ Submatrices\ *) \n\t\t\t
          submata\  = \ SubMatrix[mat, \ {1, \ 1}, \ {1, \ 1}]; \ \ \n\t\t\t
          submatb\  = \ \ SubMatrix[mat, \ {1, \ 2}, \ {1, \ n - 1}]; \n\t\t\t
          submatc\  = \ \ SubMatrix[mat, \ {2, \ 1}, \ {n - 1, \ 1}]; \ \n
          \t\t\tsubmatd\  = \ \ 
            SubMatrix[mat, \ {2, \ 2}, \ {n - 1, \ n - 1}]; \n
          \t\t\t (*\ Recursive\ steps\ on\ matrices\ size\ n - 1\ *) \n\t\t\t
          dtil\  = \ 
            submatd - 
              MatMult[submatc, {{\ inv[\ submata[\([1, 1]\)]\ ]\ }}, 
                submatb]; \n\t\t\tsubgauss\  = \ NGaussElimination[dtil]; \n
          \t\t\tcinvsubmata = \ 
            MatMult[submatc\ , \ {{inv[submata[\([1, 1]\)]\ ]}}\ ]; \n\t\t\t
          binvsubmata = \ 
            MatMult[{{inv[submata[\([1, 1]\)]\ ]}}\ , \ submatb]; \n
          \t\t\t (*\ Lower\ Matrix\ *) \n\t\t\t
          lleft\  = \ 
            NCSimplifyRational[
              BlockMatrix[\n
                \t\t\t\t\t\t{{\ {{1}}, \ ZeroMatrix[1, n - 1]}, {cinvsubmata, 
                    \ subgauss[\([1]\)]\ }}\ \ ]]; \n
          \t\t\t (*\ Upper\ matrix\ *) \n\t\t\t
          rright\  = \ 
            NCSimplifyRational[
              BlockMatrix[\n
                \t\t\t\t\t\t{{\ {{1}}, binvsubmata}, {\ ZeroMatrix[n - 1, 1], 
                    \ subgauss[\([3]\)]\ }}\ \ ]]; \t\n
          \t\t\t (*\ Diagobal\ Matrix\ *) \n\t\t\t
          diagon\  = \ \ 
            NCSimplifyRational[
              BlockMatrix[\n
                \t\t\t\t\t\t{{\ submata, \ ZeroMatrix[1, n - 1]\ }, {\ 
                    ZeroMatrix[n - 1, 1], \ subgauss[\([2]\)]\ }}\ \ ]]; \n
          \t\t\t (*\ Results\ *) \n\t\t
          output\  = \ {lleft, \ diagon, rright}; \n\t\tReturn[output]; \t]]; 
    \)\)], "Input"],

Cell[BoxData[
    \( (*\ Without\ NCSimplifyRational\ *) \)], "Input"],

Cell[BoxData[
    \(\(NGaussElimination[mat_] := 
      Module[{output, n, \ submata, submatb, submatc, submatd, subgauss}, \ \n
        \ \ \ \ \ n = \(Dimensions[mat]\)[[1]]; \n\t\t
        If[n < 3, \ \n\t\t\tReturn[GaussElimination[mat]], \ 
          \n (*\ Else\ *) \n\t\t\t (*\ Submatrices\ *) \n\t\t\t
          submata\  = \ SubMatrix[mat, \ {1, \ 1}, \ {1, \ 1}]; \ \ \n\t\t\t
          submatb\  = \ \ SubMatrix[mat, \ {1, \ 2}, \ {1, \ n - 1}]; \n\t\t\t
          submatc\  = \ \ SubMatrix[mat, \ {2, \ 1}, \ {n - 1, \ 1}]; \ \n
          \t\t\tsubmatd\  = \ \ 
            SubMatrix[mat, \ {2, \ 2}, \ {n - 1, \ n - 1}]; \n
          \t\t\t (*\ Recursive\ steps\ on\ matrices\ size\ n - 1\ *) \n\t\t\t
          dtil\  = \ 
            submatd - 
              MatMult[submatc, {{\ inv[\ submata[[1, 1]]\ ]\ }}, submatb]; \n
          \t\t\tsubgauss\  = \ NGaussElimination[dtil]; \n\t\t\t
          cinvsubmata = \ MatMult[submatc\ , \ {{inv[submata[[1, 1]]\ ]}}\ ]; 
          \n\t\t\tbinvsubmata = \ 
            MatMult[{{inv[submata[[1, 1]]\ ]}}\ , \ submatb]; \n
          \t\t\t (*\ Lower\ Matrix\ *) \n\t\t\t
          lleft\  = \ 
            BlockMatrix[\n
              \t\t\t\t\t\t{{\ {{1}}, \ ZeroMatrix[1, n - 1]}, {cinvsubmata, \ 
                  subgauss[[1]]\ }}\ \ ]; \n\t\t\t (*\ Upper\ matrix\ *) \n
          \t\t\trright\  = \ 
            BlockMatrix[\n
              \t\t\t\t\t\t{{\ {{1}}, binvsubmata}, {\ ZeroMatrix[n - 1, 1], \ 
                  subgauss[[3]]\ }}\ \ ]; \n\t\t\t (*\ Diagobal\ Matrix\ *) \n
          \t\t\tdiagon\  = \ \ 
            BlockMatrix[\n
              \t\t\t\t\t\t{{\ submata, \ ZeroMatrix[1, n - 1]\ }, {\ 
                  ZeroMatrix[n - 1, 1], \ subgauss[[2]]\ }}\ \ ]; \n
          \t\t\t (*\ Results\ *) \n\t\t
          output\  = \ {lleft, \ diagon, rright}; \n\t\tReturn[output]; \ ]]; 
    \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(mate = {{a, b, 0, 0}, {0, f, 0, h}, {0, 0, k, 0}, {0, 0, 0, q}}; \n
    MatrixForm[mate]\), 
    \({ll, dd, rr}\  = \ NGaussElimination[mate]\), 
    \(saida\  = \ MatMult[ll, dd, rr]; \n
    NCSimplifyRational[saida]\  // MatrixForm\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "0", "0"},
            {"0", "f", "0", "h"},
            {"0", "0", "k", "0"},
            {"0", "0", "0", "q"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    \({{{1, 0, 0, 0}, {0, 1, 0, 0}, {0, 0, 1, 0}, {0, 0, 0, 1}}, {{a, 0, 0, 
          0}, {0, f, 0, 0}, {0, 0, k, 0}, {0, 0, 0, q}}, {{1, inv[a]**b, 0, 
          0}, {0, 1, 0, inv[f]**h}, {0, 0, 1, 0}, {0, 0, 0, 1}}}\)], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "0", "0"},
            {"0", "f", "0", "h"},
            {"0", "0", "k", "0"},
            {"0", "0", "0", "q"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(mate = {{a, b, c}, {d, e, f}, {g, h, i}}; \nMatrixForm[mate]\), 
    \({ll, dd, rr}\  = \ NGaussElimination[mate]; \nll\  // \ MatrixForm\), 
    \(dd\  // \ MatrixForm\), 
    \(rr\  // \ MatrixForm\), 
    \(saida\  = \ MatMult[ll, dd, rr]; \n
    NCSimplifyRational[saida]\  // MatrixForm\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(mate = {{a, b, c}, {tp[b], e, f}, {tp[c], tp[f], i}}; \n
    MatrixForm[mate]\), 
    \({ll, dd, rr}\  = \ NGaussElimination[mate]; \nll\  // \ MatrixForm\), 
    \(dd\  // \ MatrixForm\), 
    \(rr\  // \ MatrixForm\), 
    \(saida\  = \ MatMult[ll, dd, rr]; \n
    NCSimplifyRational[saida]\  // MatrixForm\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "c"},
            {\(tp[b]\), "e", "f"},
            {\(tp[c]\), \(tp[f]\), "i"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "0", "0"},
            {\(tp[b]**inv[a]\), "1", "0"},
            {\(tp[c]**inv[a]\), 
              \(tp[f]**inv[e - tp[b]**inv[a]**b] - 
                tp[c]**inv[a]**b**inv[e - tp[b]**inv[a]**b]\), "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "0", "0"},
            {"0", \(e - tp[b]**inv[a]**b\), "0"},
            {"0", "0", 
              \(i - tp[c]**inv[a]**c - tp[f]**inv[e - tp[b]**inv[a]**b]**f + 
                tp[c]**inv[a]**b**inv[e - tp[b]**inv[a]**b]**f + 
                tp[f]**inv[e - tp[b]**inv[a]**b]**tp[b]**inv[a]**c - 
                tp[c]**inv[a]**b**inv[e - tp[b]**inv[a]**b]**tp[b]**inv[a]**c
                  \)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", \(inv[a]**b\), \(inv[a]**c\)},
            {"0", "1", 
              \(inv[e - tp[b]**inv[a]**b]**f - 
                inv[e - tp[b]**inv[a]**b]**tp[b]**inv[a]**c\)},
            {"0", "0", "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "c"},
            {\(tp[b]\), "e", "f"},
            {\(tp[c]\), \(tp[f]\), "i"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(mate = {{a, b, c, d}, {e, f, g, h}, {i, j, l, m}, {nn, o, p, q}}; \n
    MatrixForm[mate]\), 
    \({ll, dd, rr}\  = \ NGaussElimination[mate]; \nll\  // \ MatrixForm\), 
    \(dd\  // \ MatrixForm\), 
    \(rr\  // \ MatrixForm\), 
    \(saida\  = MatMult[ll, dd, rr]; \n
    NCSimplifyRational[saida]\  // MatrixForm\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "c", "d"},
            {"e", "f", "g", "h"},
            {"i", "j", "l", "m"},
            {"nn", "o", "p", "q"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "0", "0", "0"},
            {\(e**inv[a]\), "1", "0", "0"},
            {\(i**inv[a]\), 
              \(\((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]\), "1", "0"},
            {\(nn\ inv[a]\), 
              \(\((o - nn\ inv[a]**b)\)**inv[f - e**inv[a]**b]\), 
              \(\((p - nn\ inv[a]**c - 
                    \((o - nn\ inv[a]**b)\)**inv[f - e**inv[a]**b]**
                      \((g - e**inv[a]**c)\))\)**
                inv[l - i**inv[a]**c - 
                    \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                      \((g - e**inv[a]**c)\)]\), "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "0", "0", "0"},
            {"0", \(f - e**inv[a]**b\), "0", "0"},
            {"0", "0", 
              \(l - i**inv[a]**c - 
                \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                  \((g - e**inv[a]**c)\)\), "0"},
            {"0", "0", "0", 
              \(q - nn\ inv[a]**d - 
                \((o - nn\ inv[a]**b)\)**inv[f - e**inv[a]**b]**
                  \((h - e**inv[a]**d)\) - 
                \((p - nn\ inv[a]**c - 
                      \((o - nn\ inv[a]**b)\)**inv[f - e**inv[a]**b]**
                        \((g - e**inv[a]**c)\))\)**
                  inv[l - i**inv[a]**c - 
                      \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                        \((g - e**inv[a]**c)\)]**
                  \((m - i**inv[a]**d - 
                      \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                        \((h - e**inv[a]**d)\))\)\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", \(inv[a]**b\), \(inv[a]**c\), \(inv[a]**d\)},
            {"0", "1", \(inv[f - e**inv[a]**b]**\((g - e**inv[a]**c)\)\), 
              \(inv[f - e**inv[a]**b]**\((h - e**inv[a]**d)\)\)},
            {"0", "0", "1", 
              \(inv[l - i**inv[a]**c - 
                    \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                      \((g - e**inv[a]**c)\)]**
                \((m - i**inv[a]**d - 
                    \((j - i**inv[a]**b)\)**inv[f - e**inv[a]**b]**
                      \((h - e**inv[a]**d)\))\)\)},
            {"0", "0", "0", "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a", "b", "c", "d"},
            {"e", "f", "g", "h"},
            {"i", "j", "l", "m"},
            {"nn", "o", "p", "q"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\nmate = {{a1, a2, a3, a4, a5, a6, a7}, \n
        \t\t\t\t\t\t\t{b1, b2, b3, b4, b5, b6, b7}, \n
        \t\t\t\t\t\t\ \ {c1, c2, c3, c4, c5, c6, c7}, \n
        \t\t\t\t\t\t\t{d1, d2, d3, d4, d5, d6, d7}, \n
        \t\t\t\t\t\t\t{e1, e2, e3, e4, e5, e6, e7}, \n
        \t\t\t\t\t\t\t{f1, f2, f3, f4, f5, f6, f7}, \n
        \t\t\t\t\t\t\t{g1, g2, g3, g4, g5, g6, g7}}; \nMatrixForm[mate]\), 
    \(\({ll, dd, rr}\  = \ NGaussElimination[mate]; \)\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a1", "a2", "a3", "a4", "a5", "a6", "a7"},
            {"b1", "b2", "b3", "b4", "b5", "b6", "b7"},
            {"c1", "c2", "c3", "c4", "c5", "c6", "c7"},
            {"d1", "d2", "d3", "d4", "d5", "d6", "d7"},
            {"e1", "e2", "e3", "e4", "e5", "e6", "e7"},
            {"f1", "f2", "f3", "f4", "f5", "f6", "f7"},
            {"g1", "g2", "g3", "g4", "g5", "g6", "g7"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(ll\  // \ MatrixForm\), 
    \(dd\  // \ MatrixForm\), 
    \(rr\  // \ MatrixForm\), 
    \(saida\  = MatMult[ll, dd, rr]; \n
    NCSimplifyRational[saida]\  // MatrixForm\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "0", "0", "0", "0", "0", "0"},
            {\(b1\ inv[a1]\), "1", "0", "0", "0", "0", "0"},
            {\(c1\ inv[a1]\), 
              \(\((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]\), "1", 
              "0", "0", "0", "0"},
            {\(d1\ inv[a1]\), 
              \(\((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]\), 
              \(\((d3 - a3\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\))\)**
                inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]\), "1", "0", "0", "0"},
            {\(e1\ inv[a1]\), 
              \(\((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]\), 
              \(\((e3 - a3\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\))\)**
                inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]\), 
              \(\((e4 - a4\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\))\)**
                inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]\), "1", "0", "0"},
            {\(f1\ inv[a1]\), 
              \(\((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]\), 
              \(\((f3 - a3\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\))\)**
                inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]\), 
              \(\((f4 - a4\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((f3 - a3\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\))\)**
                inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]\), 
              \(\((f5 - a5\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((f3 - a3\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((f4 - a4\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[
                              c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                inv[e5 - a5\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\)]\), "1", 
              "0"},
            {\(g1\ inv[a1]\), 
              \(\((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]\), 
              \(\((g3 - a3\ g1\ inv[a1] - 
                    \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\))\)**
                inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]\), 
              \(\((g4 - a4\ g1\ inv[a1] - 
                    \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((g3 - a3\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\))\)**
                inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]\), 
              \(\((g5 - a5\ g1\ inv[a1] - 
                    \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((g3 - a3\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((g4 - a4\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((g3 - a3\ g1\ inv[a1] - 
                                \((g2 - a2\ g1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                inv[e5 - a5\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\)]\), 
              \(\((g6 - a6\ g1\ inv[a1] - 
                    \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\) - 
                    \((g3 - a3\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c6 - a6\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\))\) - 
                    \((g4 - a4\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((g3 - a3\ g1\ inv[a1] - 
                                \((g2 - a2\ g1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d6 - a6\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                    \((g5 - a5\ g1\ inv[a1] - 
                          \((g2 - a2\ g1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((g3 - a3\ g1\ inv[a1] - 
                                \((g2 - a2\ g1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((g4 - a4\ g1\ inv[a1] - 
                                \((g2 - a2\ g1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((g3 - a3\ g1\ inv[a1] - 
                                      \((g2 - a2\ g1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                      inv[e5 - a5\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                      \((e6 - a6\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d6 - a6\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c6 - a6\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b6 - a6\ b1\ inv[a1])\))\))\))\))
                  \)**
                inv[f6 - a6\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\) - 
                    \((f3 - a3\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c6 - a6\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\))\) - 
                    \((f4 - a4\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d6 - a6\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                    \((f5 - a5\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((f4 - a4\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((f3 - a3\ f1\ inv[a1] - 
                                      \((f2 - a2\ f1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                      inv[e5 - a5\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                      \((e6 - a6\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[
                              c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d6 - a6\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c6 - a6\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b6 - a6\ b1\ inv[a1])\))\))\))\)]
                  \), "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a1", "0", "0", "0", "0", "0", "0"},
            {"0", \(b2 - a2\ b1\ inv[a1]\), "0", "0", "0", "0", "0"},
            {"0", "0", 
              \(c3 - a3\ c1\ inv[a1] - 
                \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                  \((b3 - a3\ b1\ inv[a1])\)\), "0", "0", "0", "0"},
            {"0", "0", "0", 
              \(d4 - a4\ d1\ inv[a1] - 
                \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                  \((b4 - a4\ b1\ inv[a1])\) - 
                \((d3 - a3\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\))\)**
                  inv[c3 - a3\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\)]**
                  \((c4 - a4\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\))\)\), "0", "0", "0"},
            {"0", "0", "0", "0", 
              \(e5 - a5\ e1\ inv[a1] - 
                \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                  \((b5 - a5\ b1\ inv[a1])\) - 
                \((e3 - a3\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\))\)**
                  inv[c3 - a3\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\)]**
                  \((c5 - a5\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\))\) - 
                \((e4 - a4\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((e3 - a3\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\))\)**
                  inv[d4 - a4\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\)]**
                  \((d5 - a5\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c5 - a5\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\))\))\)\), "0", "0"},
            {"0", "0", "0", "0", "0", 
              \(f6 - a6\ f1\ inv[a1] - 
                \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                  \((b6 - a6\ b1\ inv[a1])\) - 
                \((f3 - a3\ f1\ inv[a1] - 
                      \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\))\)**
                  inv[c3 - a3\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\)]**
                  \((c6 - a6\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b6 - a6\ b1\ inv[a1])\))\) - 
                \((f4 - a4\ f1\ inv[a1] - 
                      \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((f3 - a3\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\))\)**
                  inv[d4 - a4\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\)]**
                  \((d6 - a6\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b6 - a6\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c6 - a6\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                \((f5 - a5\ f1\ inv[a1] - 
                      \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\) - 
                      \((f3 - a3\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c5 - a5\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\))\) - 
                      \((f4 - a4\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((f3 - a3\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[
                                c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d5 - a5\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                  inv[e5 - a5\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\) - 
                      \((e3 - a3\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c5 - a5\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\))\) - 
                      \((e4 - a4\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d5 - a5\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                  \((e6 - a6\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b6 - a6\ b1\ inv[a1])\) - 
                      \((e3 - a3\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c6 - a6\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\))\) - 
                      \((e4 - a4\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d6 - a6\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c6 - a6\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\))\))\))\)\), 
              "0"},
            {"0", "0", "0", "0", "0", "0", 
              \(g7 - a7\ g1\ inv[a1] - 
                \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                  \((b7 - a7\ b1\ inv[a1])\) - 
                \((g3 - a3\ g1\ inv[a1] - 
                      \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\))\)**
                  inv[c3 - a3\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b3 - a3\ b1\ inv[a1])\)]**
                  \((c7 - a7\ c1\ inv[a1] - 
                      \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b7 - a7\ b1\ inv[a1])\))\) - 
                \((g4 - a4\ g1\ inv[a1] - 
                      \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((g3 - a3\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\))\)**
                  inv[d4 - a4\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b4 - a4\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c4 - a4\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\))\)]**
                  \((d7 - a7\ d1\ inv[a1] - 
                      \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b7 - a7\ b1\ inv[a1])\) - 
                      \((d3 - a3\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c7 - a7\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\))\))\) - 
                \((g5 - a5\ g1\ inv[a1] - 
                      \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\) - 
                      \((g3 - a3\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c5 - a5\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\))\) - 
                      \((g4 - a4\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((g3 - a3\ g1\ inv[a1] - 
                                  \((g2 - a2\ g1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d5 - a5\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                  inv[e5 - a5\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b5 - a5\ b1\ inv[a1])\) - 
                      \((e3 - a3\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c5 - a5\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\))\) - 
                      \((e4 - a4\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d5 - a5\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                  \((e7 - a7\ e1\ inv[a1] - 
                      \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b7 - a7\ b1\ inv[a1])\) - 
                      \((e3 - a3\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c7 - a7\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\))\) - 
                      \((e4 - a4\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d7 - a7\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c7 - a7\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b7 - a7\ b1\ inv[a1])\))\))\))\) - 
                \((g6 - a6\ g1\ inv[a1] - 
                      \((g2 - a2\ g1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b6 - a6\ b1\ inv[a1])\) - 
                      \((g3 - a3\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c6 - a6\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\))\) - 
                      \((g4 - a4\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((g3 - a3\ g1\ inv[a1] - 
                                  \((g2 - a2\ g1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d6 - a6\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[
                                c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c6 - a6\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                      \((g5 - a5\ g1\ inv[a1] - 
                            \((g2 - a2\ g1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((g3 - a3\ g1\ inv[a1] - 
                                  \((g2 - a2\ g1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((g4 - a4\ g1\ inv[a1] - 
                                  \((g2 - a2\ g1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((g3 - a3\ g1\ inv[a1] - 
                                        \((g2 - a2\ g1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                        inv[e5 - a5\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                        \((e6 - a6\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c6 - a6\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d6 - a6\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c6 - a6\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b6 - a6\ b1\ inv[a1])\))\))\))\))
                    \)**
                  inv[f6 - a6\ f1\ inv[a1] - 
                      \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b6 - a6\ b1\ inv[a1])\) - 
                      \((f3 - a3\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c6 - a6\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\))\) - 
                      \((f4 - a4\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((f3 - a3\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d6 - a6\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c6 - a6\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                      \((f5 - a5\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((f3 - a3\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((f4 - a4\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((f3 - a3\ f1\ inv[a1] - 
                                        \((f2 - a2\ f1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                        inv[e5 - a5\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                        \((e6 - a6\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b6 - a6\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[
                                c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c6 - a6\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d6 - a6\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b6 - a6\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c6 - a6\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b6 - a6\ b1\ inv[a1])\))\))\))\)]**
                  \((f7 - a7\ f1\ inv[a1] - 
                      \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                        \((b7 - a7\ b1\ inv[a1])\) - 
                      \((f3 - a3\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\))\)**
                        inv[c3 - a3\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b3 - a3\ b1\ inv[a1])\)]**
                        \((c7 - a7\ c1\ inv[a1] - 
                            \((c2 - a2\ c1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\))\) - 
                      \((f4 - a4\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((f3 - a3\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\))\)**
                        inv[d4 - a4\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b4 - a4\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c4 - a4\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\))\)]**
                        \((d7 - a7\ d1\ inv[a1] - 
                            \((d2 - a2\ d1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\) - 
                            \((d3 - a3\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c7 - a7\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b7 - a7\ b1\ inv[a1])\))\))\) - 
                      \((f5 - a5\ f1\ inv[a1] - 
                            \((f2 - a2\ f1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((f3 - a3\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((f4 - a4\ f1\ inv[a1] - 
                                  \((f2 - a2\ f1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((f3 - a3\ f1\ inv[a1] - 
                                        \((f2 - a2\ f1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                        inv[e5 - a5\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b5 - a5\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c5 - a5\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d5 - a5\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b5 - a5\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c5 - a5\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                        \((e7 - a7\ e1\ inv[a1] - 
                            \((e2 - a2\ e1\ inv[a1])\)**
                              inv[b2 - a2\ b1\ inv[a1]]**
                              \((b7 - a7\ b1\ inv[a1])\) - 
                            \((e3 - a3\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\))\)**
                              inv[c3 - a3\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b3 - a3\ b1\ inv[a1])\)]**
                              \((c7 - a7\ c1\ inv[a1] - 
                                  \((c2 - a2\ c1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b7 - a7\ b1\ inv[a1])\))\) - 
                            \((e4 - a4\ e1\ inv[a1] - 
                                  \((e2 - a2\ e1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((e3 - a3\ e1\ inv[a1] - 
                                        \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                              inv[d4 - a4\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b4 - a4\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c4 - a4\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                              \((d7 - a7\ d1\ inv[a1] - 
                                  \((d2 - a2\ d1\ inv[a1])\)**
                                    inv[b2 - a2\ b1\ inv[a1]]**
                                    \((b7 - a7\ b1\ inv[a1])\) - 
                                  \((d3 - a3\ d1\ inv[a1] - 
                                        \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                    inv[c3 - a3\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                    \((c7 - a7\ c1\ inv[a1] - 
                                        \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b7 - a7\ b1\ inv[a1])\))\))\))\))
                    \)\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", \(a2\ inv[a1]\), \(a3\ inv[a1]\), \(a4\ inv[a1]\), 
              \(a5\ inv[a1]\), \(a6\ inv[a1]\), \(a7\ inv[a1]\)},
            {"0", "1", 
              \(inv[b2 - a2\ b1\ inv[a1]]**\((b3 - a3\ b1\ inv[a1])\)\), 
              \(inv[b2 - a2\ b1\ inv[a1]]**\((b4 - a4\ b1\ inv[a1])\)\), 
              \(inv[b2 - a2\ b1\ inv[a1]]**\((b5 - a5\ b1\ inv[a1])\)\), 
              \(inv[b2 - a2\ b1\ inv[a1]]**\((b6 - a6\ b1\ inv[a1])\)\), 
              \(inv[b2 - a2\ b1\ inv[a1]]**\((b7 - a7\ b1\ inv[a1])\)\)},
            {"0", "0", "1", 
              \(inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]**
                \((c4 - a4\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\))\)\), 
              \(inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]**
                \((c5 - a5\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\))\)\), 
              \(inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]**
                \((c6 - a6\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\))\)\), 
              \(inv[c3 - a3\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b3 - a3\ b1\ inv[a1])\)]**
                \((c7 - a7\ c1\ inv[a1] - 
                    \((c2 - a2\ c1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b7 - a7\ b1\ inv[a1])\))\)\)},
            {"0", "0", "0", "1", 
              \(inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]**
                \((d5 - a5\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\))\)\), 
              \(inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]**
                \((d6 - a6\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c6 - a6\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\))\))\)\), 
              \(inv[d4 - a4\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b4 - a4\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c4 - a4\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\))\)]**
                \((d7 - a7\ d1\ inv[a1] - 
                    \((d2 - a2\ d1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b7 - a7\ b1\ inv[a1])\) - 
                    \((d3 - a3\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c7 - a7\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\))\))\)\)},
            {"0", "0", "0", "0", "1", 
              \(inv[e5 - a5\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                \((e6 - a6\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c6 - a6\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d6 - a6\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\))\))\)\), 
              \(inv[e5 - a5\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b5 - a5\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c5 - a5\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d5 - a5\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                \((e7 - a7\ e1\ inv[a1] - 
                    \((e2 - a2\ e1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b7 - a7\ b1\ inv[a1])\) - 
                    \((e3 - a3\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c7 - a7\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\))\) - 
                    \((e4 - a4\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d7 - a7\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c7 - a7\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b7 - a7\ b1\ inv[a1])\))\))\))\)\)},
            {"0", "0", "0", "0", "0", "1", 
              \(inv[f6 - a6\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b6 - a6\ b1\ inv[a1])\) - 
                    \((f3 - a3\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c6 - a6\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\))\) - 
                    \((f4 - a4\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d6 - a6\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\))\) - 
                    \((f5 - a5\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((f4 - a4\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((f3 - a3\ f1\ inv[a1] - 
                                      \((f2 - a2\ f1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                      inv[e5 - a5\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                      \((e6 - a6\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b6 - a6\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c6 - a6\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d6 - a6\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b6 - a6\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c6 - a6\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b6 - a6\ b1\ inv[a1])\))\))\))\)]**
                \((f7 - a7\ f1\ inv[a1] - 
                    \((f2 - a2\ f1\ inv[a1])\)**inv[b2 - a2\ b1\ inv[a1]]**
                      \((b7 - a7\ b1\ inv[a1])\) - 
                    \((f3 - a3\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\))\)**
                      inv[c3 - a3\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b3 - a3\ b1\ inv[a1])\)]**
                      \((c7 - a7\ c1\ inv[a1] - 
                          \((c2 - a2\ c1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\))\) - 
                    \((f4 - a4\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\))\)**
                      inv[d4 - a4\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b4 - a4\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c4 - a4\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\))\)]**
                      \((d7 - a7\ d1\ inv[a1] - 
                          \((d2 - a2\ d1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\) - 
                          \((d3 - a3\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c7 - a7\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b7 - a7\ b1\ inv[a1])\))\))\) - 
                    \((f5 - a5\ f1\ inv[a1] - 
                          \((f2 - a2\ f1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((f3 - a3\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((f4 - a4\ f1\ inv[a1] - 
                                \((f2 - a2\ f1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((f3 - a3\ f1\ inv[a1] - 
                                      \((f2 - a2\ f1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\))\)**
                      inv[e5 - a5\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b5 - a5\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[
                              c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c5 - a5\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d5 - a5\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b5 - a5\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c5 - a5\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b5 - a5\ b1\ inv[a1])\))\))\)]**
                      \((e7 - a7\ e1\ inv[a1] - 
                          \((e2 - a2\ e1\ inv[a1])\)**
                            inv[b2 - a2\ b1\ inv[a1]]**
                            \((b7 - a7\ b1\ inv[a1])\) - 
                          \((e3 - a3\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\))\)**
                            inv[c3 - a3\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b3 - a3\ b1\ inv[a1])\)]**
                            \((c7 - a7\ c1\ inv[a1] - 
                                \((c2 - a2\ c1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b7 - a7\ b1\ inv[a1])\))\) - 
                          \((e4 - a4\ e1\ inv[a1] - 
                                \((e2 - a2\ e1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((e3 - a3\ e1\ inv[a1] - 
                                      \((e2 - a2\ e1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\))\)**
                            inv[d4 - a4\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b4 - a4\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c4 - a4\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b4 - a4\ b1\ inv[a1])\))\)]**
                            \((d7 - a7\ d1\ inv[a1] - 
                                \((d2 - a2\ d1\ inv[a1])\)**
                                  inv[b2 - a2\ b1\ inv[a1]]**
                                  \((b7 - a7\ b1\ inv[a1])\) - 
                                \((d3 - a3\ d1\ inv[a1] - 
                                      \((d2 - a2\ d1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\))\)**
                                  inv[c3 - a3\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b3 - a3\ b1\ inv[a1])\)]**
                                  \((c7 - a7\ c1\ inv[a1] - 
                                      \((c2 - a2\ c1\ inv[a1])\)**
                                        inv[b2 - a2\ b1\ inv[a1]]**
                                        \((b7 - a7\ b1\ inv[a1])\))\))\))\))
                  \)\)},
            {"0", "0", "0", "0", "0", "0", "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"a1", "a2", "a3", "a4", "a5", "a6", "a7"},
            {"b1", "b2", "b3", "b4", "b5", "b6", "b7"},
            {"c1", "c2", "c3", "c4", "c5", "c6", "c7"},
            {"d1", "d2", "d3", "d4", "d5", "d6", "d7"},
            {"e1", "e2", "e3", "e4", "e5", "e6", "e7"},
            {"f1", "f2", "f3", "f4", "f5", "f6", "f7"},
            {"g1", "g2", "g3", "g4", "g5", "g6", "g7"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \({ll, dd, rr}\  = \ 
      NGaussElimination[{{x, y, z}, {t, 0, u}, {a, b, c}}]; \n
    ll\  // \ MatrixForm\), 
    \(dd\  // \ MatrixForm\), 
    \(rr\  // \ MatrixForm\n\t\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "0", "0"},
            {\(t**inv[x]\), "1", "0"},
            {\(a**inv[x]\), \(a**inv[t] - b**inv[y]**x**inv[t]\), "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"x", "0", "0"},
            {"0", \(-t**inv[x]**y\), "0"},
            {"0", "0", 
              \(c - a**inv[t]**u - b**inv[y]**z + b**inv[y]**x**inv[t]**u\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", \(inv[x]**y\), \(inv[x]**z\)},
            {"0", "1", \(inv[y]**z - inv[y]**x**inv[t]**u\)},
            {"0", "0", "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{956, 593},
WindowMargins->{{98, Automatic}, {130, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 120, 2, 43, "Input"],

Cell[CellGroupData[{
Cell[1854, 55, 63, 1, 27, "Input"],
Cell[1920, 58, 200, 3, 39, "Message"]
}, Open  ]],
Cell[2135, 64, 121, 2, 43, "Input"],
Cell[2259, 68, 98, 2, 43, "Input"],
Cell[2360, 72, 70, 1, 27, "Input"],
Cell[2433, 75, 2076, 40, 443, "Input"],
Cell[4512, 117, 70, 1, 27, "Input"],
Cell[4585, 120, 1886, 34, 443, "Input"],

Cell[CellGroupData[{
Cell[6496, 158, 268, 5, 91, "Input"],
Cell[6767, 165, 249, 8, 80, "Output"],
Cell[7019, 175, 246, 3, 43, "Output"],
Cell[7268, 180, 249, 8, 80, "Output"]
}, Open  ]],
Cell[7532, 191, 326, 6, 139, "Input"],

Cell[CellGroupData[{
Cell[7883, 201, 343, 7, 139, "Input"],
Cell[8229, 210, 218, 7, 62, "Output"],
Cell[8450, 219, 339, 9, 62, "Output"],
Cell[8792, 230, 531, 12, 119, "Output"],
Cell[9326, 244, 327, 9, 62, "Output"],
Cell[9656, 255, 218, 7, 62, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9911, 267, 353, 7, 139, "Input"],
Cell[10267, 276, 250, 8, 80, "Output"],
Cell[10520, 286, 715, 16, 160, "Output"],
Cell[11238, 304, 1037, 23, 160, "Output"],
Cell[12278, 329, 696, 15, 160, "Output"],
Cell[12977, 346, 250, 8, 80, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13264, 359, 485, 8, 171, "Input"],
Cell[13752, 369, 505, 11, 83, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14294, 385, 207, 5, 55, "Input"],
Cell[14504, 392, 53919, 898, 185, "Output"],
Cell[68426, 1292, 83132, 1346, 185, "Output"],
Cell[151561, 2640, 53854, 895, 185, "Output"],
Cell[205418, 3537, 505, 11, 134, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[205960, 3553, 208, 5, 91, "Input"],
Cell[206171, 3560, 253, 7, 62, "Output"],
Cell[206427, 3569, 287, 8, 62, "Output"],
Cell[206717, 3579, 253, 7, 62, "Output"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

